@@ -24,7 +24,7 @@ You can use and redistribute Perlbal under the same terms as Perl itself.
 package Perlbal;
 
 use vars qw($VERSION);
-$VERSION = '1.45';
+$VERSION = '1.46';
 
 use constant DEBUG => $ENV{PERLBAL_DEBUG} || 0;
 use constant DEBUG_OBJ => $ENV{PERLBAL_DEBUG_OBJ} || 0;
@@ -1101,7 +1101,7 @@ sub log {
         printf(shift(@_) . "\n", @_);
     } else {
         # just pass the parameters to syslog
-        syslog(@_) if $Perlbal::SYSLOG_AVAILABLE;
+        Sys::Syslog::syslog(@_) if $Perlbal::SYSLOG_AVAILABLE;
     }
 }
 
